defmodule DecentralisedBookIndex.Metadata.Author do
  use Ash.Resource,
    otp_app: :decentralised_book_index,
    domain: DecentralisedBookIndex.Metadata,
    data_layer: AshPostgres.DataLayer

  postgres do
    table "authors"
    repo DecentralisedBookIndex.Repo
  end

  actions do
    defaults [:read, :update, :destroy]

    update :add_author_to_alias_registry do
      accept [:author_alias_registry_id]
    end

    create :create do
      primary? true
      accept [:name, :description, :author_alias_registry_id]

      change fn changeset, _ ->
        registry_id = Ash.Changeset.get_attribute(changeset, :author_alias_registry_id)

        if registry_id == nil do
          {:ok, registry} = DecentralisedBookIndex.Metadata.create_author_alias_registry()

          Ash.Changeset.force_change_attribute(changeset, :author_alias_registry_id, registry.id)
        else
          changeset
        end
      end
    end

    read :by_id do
      argument :id, :uuid, allow_nil?: false
      get? true
      filter expr(id == ^arg(:id))
    end

    end
  end

  attributes do
    uuid_primary_key :id

    attribute :name, :string do
      allow_nil? false
      public? true
    end

    attribute :description, :string do
      allow_nil? false
      public? true
    end

    timestamps()
  end

  relationships do
    belongs_to :author_alias_registry, DecentralisedBookIndex.Metadata.AuthorAliasRegistry do
      attribute_writable? true
    end
  end
end
